# source the data simulation code
source("simulation.R")
# source the model estimation code
source("estimation.R")
# compile the fortran code files (.f) first to get the .so files to be loaded into R:
# R CMD SHLIB double_ker_epa.f
# R CMD SHLIB linear_inter.f
dyn.load("double_ker_epa.so")
dyn.load("linear_inter.so")

# for illustration purpose, simulate an example dataset under the first scenario in Table 1 in the paper
N=100
data <- sim.data_two_am(N=N, beta=0.5,gamma=0.2, mprob=1, sce="t",tau=20,var=0.2,frailty="gamma")
b.carr <- data[[1]]
b.phar <- data[[2]]

# estimation results from method LCCF
lccf <- est_add_mul(b.carr, b.phar, type="ACCF", N=N, tau=20)

# estimation results from method linear interpolation
linear <- est_add_mul(b.carr, b.phar, type="interp", N=N, tau=20)

# estimation results from the proposed kernel smoothing method
bandwidth_con <- select.con_add_mul(b.carr, low=0.5, up=20,N=N,tau=20)
kernel <- est_add_mul(b.carr, b.phar, type="kernel", N=N, tau=20, bandwidth = bandwidth_con*(N^(-1/3)))
seker <- boot_add_mul(b.carr, b.phar, nb=50, bandwidth=bandwidth_con*(N^(-1/3)), type="kernel", N=N, tau=20)
# indicators for whether the 95% CI covers the true value
cpker_b1=ifelse((kernel[1]-1.96*seker[1]) < 0.5 & 0.5 < (kernel[1]+1.96*seker[1]),1,0)
cpker_b2=ifelse((kernel[2]-1.96*seker[2]) < 0.2 & 0.2 < (kernel[2]+1.96*seker[2]),1,0)

