library(MASS)

#the continuous covariate
#compute the covariate value at certain time mt with random intercept b0 and random slope b1.
g.dat_am <- function(mt, b0, b1){ b0+b1*mt } 

#generate the binary covariate process
#rr1 is the value of g(t) in 0-10 and rr2 is the value of g(t) in 10-20.
gencov_b_am <- function(N, rr1, rr2, lc, tau ){
  
  sdata <-NULL
  for(i in 1:N){
    z <- rgamma(1, shape=4, scale=1/4)
    # 	z <- exp(rnorm(1,0,0.5))
    #z<-1
    #w is the baseline
    w <- rbinom(1,1,1/(rr1+1))
    tmp <- w
    gap <-gas<- NULL
    lg<-0; f<-1
    
    while(lg<tau/2){
      m <- ifelse (tmp %%2 ==0, rr1, 1) 
      gap <- c(gap, rexp(1, 1/(z*m*lc))) 
      gas <- c(gas, as.integer(tmp %%2 ==1))
      tmp <- tmp+1  
      lg <- sum(gap)
    }
    
    gap[length(gap)] <- tau/2-sum(gap[-length(gap)] )    
    lg1 <- 0; gap1 <- NULL; gas1 <- NULL;
    tmp <- tmp-1 
    while(lg1<tau/2){
      m <- ifelse (tmp %%2 ==0, rr2, 1) 
      gap1 <- c(gap1, rexp(1, 1/(z*m*lc))) 
      gas1 <- c(gas1, as.integer(tmp %%2 ==1))
      tmp <- tmp+1  
      lg1 <- sum(gap1)
    }
    gap1[length(gap1)] <- tau/2-sum(gap1[-length(gap1)] )    
    
    #the last one of gap and the first one of gap1 have the same covariate value
    gap[length(gap)] <- gap[length(gap)] +gap1[1]
    gap <- c(gap, gap1[-1])
    gas <- c(gas, gas1[-1])   
    sdata <-rbind(sdata, cbind(id=i, gap=gap, gas=gas))
  } 
  
  sdata <- data.frame(sdata)
  sdata$time <-unlist(tapply(sdata$gap,sdata$id,cumsum))
  sdata
}

#generate event time and the observation of covariate at the event times
genevent_two_am <- function(k, data1, b0, b1, beta,gamma, lam0, lam1, tau, cen,var,frailty){
  #get the binary covariate information for subject k
  t<-data1$time[data1$id==k]
  x<-data1$gas[data1$id==k]
  g<-data1$gap[data1$id==k]
  #lam0, lam1 are for the baseline hazard
  #z: random effect (recurrent events generated from intensity model conditional on the subject-specific random effect)
  if (var==0) {
    uu <- 1
  } else if (frailty=="gamma"){
    uu <- rgamma(1, shape=1/var, scale=var) #mean=shape*scale
  } else if (frailty=="lognormal"){
    uu <- rlnorm(1, meanlog = -log(var+1)/2, sdlog = sqrt(log(var+1))) 
  }

  Lam <- function(s, a=0, b=beta,r=gamma, lam.0=lam0, lam.1=lam1, zz=uu, tt=t, gg=g, xx=x, tau1=tau){ 
    if (s <= tau1/2){	
      foot0 <- unique( c(tt[tt<=s], s ))
      indx2 <-  sum(tt<=(s-10^-11))
      foog0 <- foot0-c(0, foot0[-length(foot0)])
      foox0 <- xx[1:(indx2+1)]  
      Lam <- zz*(r*b0*s + 1/2*r*b1*s^2+lam.0*(exp(b)*sum(foog0[foox0==1])+sum(foog0[foox0==0])))}
    if (s > tau1/2){ 
      foot0_pre <- unique( c(tt[tt<=tau1/2], tau1/2 ))
      indx2_pre <- sum(tt<=(tau1/2-10^-11))
      foog0_pre <- foot0_pre-c(0, foot0_pre[-length(foot0_pre)])
      foox0_pre <- xx[1:(indx2_pre+1)] 
      
      foot0 <- unique( c(tt[tt>(tau1/2) & tt<=s], s ))
      indx20 <- sum(tt<=(tau1/2))
      indx2 <-  sum(tt<=(s-10^-11))
      foog0 <- foot0-c(tau1/2, foot0[-length(foot0)])
      foox0 <- xx[(indx20+1):(indx2+1)]  
      Lam <- zz*(r*b0*s+1/2*r*b1*s^2 + lam.0*(exp(b)*sum(foog0_pre[foox0_pre==1])+sum(foog0_pre[foox0_pre==0])) + lam.1*(exp(b)*sum(foog0[foox0==1])+sum(foog0[foox0==0])))}
    Lam-a
  }
  # if a=0, then function Lam outputs the cumulative hazard up to time t
  # Lam0 is the total hazard up to tau
  #Lam0=Lam(tau)
  Lam0=Lam(cen)
  m <- rpois(1, Lam0)    
  event0 <- NULL
  X1 <- NULL
  if (m>0){
    for(kk in 1:m){
      u <- runif(1)*Lam0
      #foo<- uniroot(Lam, c(10^(-10),tau), a=u)$root
      foo<- uniroot(Lam, c(10^(-10),cen), a=u)$root
      event0<- c(event0, foo) 
      X1 <- c(X1, x[foo<t][1])
    }
    event <- event0[order(event0)]
    X1 <- X1[order(event0)]
  } else { event<- -99; X1 <- -99}
  X2 <- g.dat_am(event, b0, b1)	 
  t(matrix(cbind(k,event,X1,X2), ncol=4 )   )
}


#generate observations of covariate other than those at event times
gencarr_two_am <- function(k, data1, b0, b1, tau){
  tmp <- data1[data1$id==k,]
  vis <- c(0,(0:(tau-1))+runif(tau))
  #vis <- (0:(tau-1))+runif(tau)
  #vis <- c(0,runif(4),rep(1:(tau-1),5)+runif((tau-1)*5))
  #vis <- sort(vis)
  x1 <- sapply(vis, 
               function(s, t=tmp$time, x=tmp$gas){ x[s<t][1] 
               })
  x2 <- g.dat_am(vis, b0, b1)
  t(as.matrix(cbind(id=k, time=vis, z1=x1, z2=x2)))
} 


#mprob: missing probability(1, 0.8, 0.6, 0.4)
#it is possible that among N people, some don't have events at all.
sim.data_two_am <- function(N, beta,gamma, mprob, sce, tau,var,frailty)
{
  if (sce=="t")
  {
    Mu <- c(0.5, -0.05)
    Sigma <- c(0.05, 0.0005)
    #Mu <- c(1.5,-0.05)
    rr2 <- 6
  }
  if (sce=="nt")
  {
    Mu <- c(0.5, 0)
    Sigma <- c(0.05, 0.0005)
    #Mu <- c(1.5,0)
    rr2 <- 4
  }
  #tau <- 20
  
  b0 <- rnorm(N,mean=Mu[1],sd=sqrt(Sigma[1]))
  b1 <- rnorm(N,mean=Mu[2],sd=sqrt(Sigma[2]))
  
  truez <- cbind(1:N, b0,b1)
  colnames(truez) <- c("id","b0","b1")
  
  #binary covariate
  gdata <- gencov_b_am(N=N, rr1=4, rr2=rr2, lc=1, tau=tau)
  
  g.censor = runif(N, 0, tau)
  tmp = NULL
  for (i in 1:N){
    #tmp <- cbind(tmp, genevent_two(i, data1=gdata, b0[i], b1[i],beta=beta,lam0=0.1,lam1=0.3, tau=tau))
    tmp <- cbind(tmp, genevent_two_am(i, data1=gdata, b0[i], b1[i],beta=beta,gamma=gamma,lam0=0.3,lam1=0.5, tau=tau, cen=g.censor[i],var=var,frailty=frailty))
  }	
  
  ##  tmp<-sapply(1:N, genevent,b0=2,b1=1,beta=0.5,lam0=0.05,lam1=0.2, tau=20)
  edata <- data.frame(matrix(unlist(tmp),ncol=4, byrow=TRUE, 
                             dimnames=list(NULL, c("id", "time","x","z")) ))
  edata <- edata[edata$time!=-99,]
  
  tmp =NULL
  for (i in 1:N){
    tmp <- cbind(tmp, gencarr_two_am(i, data1=gdata, b0[i], b1[i], tau=tau))
  }	 	
  cdata<-data.frame(matrix(unlist(tmp),ncol=4, byrow=TRUE, 
                           dimnames=list(NULL, c("id", "time","x","z")) ))
  
  b.phar <- edata
  #bindx <-  rbinom(5*N*tau,1,mprob)
  bindx <-  rbinom(N*(tau+1),1,mprob)
  #non-uniform missing probability
  #bindx[(1:(N*tau))%%20>10] <-rbinom(N*tau,1,mprob[bb]*0.5)
  bindx[0:(N-1)*(tau+1)+1] <- 1
  b.carr <- cdata[bindx ==1,]
  
  #censoring time is unif(10,20)
  #g.censor = rep(NA, N); for (i in 1:N){	g.censor[i] = max(b.carr$time[b.carr$id==i])} 
  #g.censor = runif(N, tau/4*3, tau) 
  #g.censor = runif(N, 0, tau)
  #b.phar$censor <- 1
  b.carr$censor <- 1
  for (i in 1:N){	b.carr$censor[b.carr$id==i][b.carr$time[b.carr$id==i]>g.censor[i]] <- 0}
  b.carr <- b.carr[b.carr$censor==1,]
  for (i in 1:N){	b.carr$cent[b.carr$id==i] <- rep(g.censor[i], nrow(b.carr[b.carr$id==i,]))}
  for (i in 1:N){	b.phar$cent[b.phar$id==i] <- rep(g.censor[i], nrow(b.phar[b.phar$id==i,]))}
  b.carr <- b.carr[,c("id","time","x","z","cent")]
  
  for (i in 1:N){	gdata$cent[gdata$id==i] <- rep(g.censor[i], nrow(gdata[gdata$id==i,]))}
  gdata <- gdata[,c("id","time","gas","cent")]
  colnames(gdata) <- c("id","time","x","cent")
  return(list(b.carr, b.phar, truez,gdata))
}

#----------------------------------------#
# Generate dataset for recurrent event analysis
# Last observation carry forward method
# Follow-up stopped at the last contact 
#----------------------------------------#

genphar_two_am <- function(carriage, pharyn, N=N, tau=tau){
  pdata <- NULL
  ids <- unique(c(carriage$id, pharyn$id))
  for( k in 1:length(ids)){
    foo <- NULL
    foo1 <- carriage[ carriage$id==ids[k],]
    foo2 <- pharyn[ pharyn$id==ids[k],]
    
    if( nrow(foo1)>0 | nrow(foo2)>0 ){
      udt <- sort(unique(c(foo1$time, foo2$time)))
      #udt <- c(udt, max(udt)+10^(-10))
      centime <- unique(foo1$cent)
      udt <- c(udt, max((max(udt)+10^(-10)),centime))
      foo <- data.frame(cbind(id=k, start=udt[-length(udt)], 
                              stop=udt[-1]))
      foo$d <- 0 # foo$d is the indicator of event
      foo$d[sapply(foo$stop, function(s, t=foo2$time) { any(s==t)})]<- 1
      foo$x <- sapply(foo$start, 
                      function(s, t1=foo1$time, x1=foo1$x, t2=foo2$time, 
                               x2=foo2$x){ 
                        if( any(s==t1)){ x1[s==t1]} else { x2[s==t2]} } )
      foo$z <- sapply(foo$start, 
                       function(s, t1=foo1$time, x1=foo1$z, t2=foo2$time, 
                                x2=foo2$z){ 
                         if( any(s==t1)){ x1[s==t1]} else { x2[s==t2]} } )
      pdata <- rbind(pdata, foo)
      
    } 
  }
  pdata
}


#################################################################
#the continuous covariate
#compute the covariate value at certain time mt with random intercept b0 and random slope b1.

#generate event time and the observation of covariate at the event times
genevent_two_am_re <- function(k, data1, b0, b1, beta,gamma, lam0, lam1, tau, cen,var,frailty){
  #get the binary covariate information for subject k
  t<-data1$time[data1$id==k]
  x<-data1$gas[data1$id==k]
  g<-data1$gap[data1$id==k]
  #lam0, lam1 are for the baseline hazard
  #z: random effect (recurrent events generated from intensity model conditional on the subject-specific random effect)
  if (var==0) {
    uu <- 1
  } else if (frailty=="gamma"){
    uu <- rgamma(1, shape=1/var, scale=var) #mean=shape*scale
  } else if (frailty=="lognormal"){
    uu <- rlnorm(1, meanlog = -log(var+1)/2, sdlog = sqrt(log(var+1))) 
  }
  
  Lam <- function(s, a=0, b=beta,r=gamma, lam.0=lam0, lam.1=lam1, zz=uu, tt=t, gg=g, xx=x, tau1=tau){ 
    if (s <= tau1/2){	
      foot0 <- unique( c(tt[tt<=s], s ))
      indx2 <-  sum(tt<=(s-10^-11))
      foog0 <- foot0-c(0, foot0[-length(foot0)])
      foox0 <- xx[1:(indx2+1)] 
      if (b==0) {
        Lam <- zz*(r*sum(foog0[foox0==1])+lam.0*s)
      } else {
      Lam <- zz*(r*sum(foog0[foox0==1])+lam.0*exp(b*b0)/(b*b1)*(exp(b*b1*s)-1))
      }
      }
    if (s > tau1/2){ 
      foot0 <- unique( c(tt[tt<=s], s ))
      indx2 <-  sum(tt<=(s-10^-11))
      foog0 <- foot0-c(0, foot0[-length(foot0)])
      foox0 <- xx[1:(indx2+1)] 
      if (b==0) {
        Lam <- zz*(r*sum(foog0[foox0==1])+lam.0*tau1/2 + lam.1*(s-tau1/2))
      } else {
      Lam <- zz*(r*sum(foog0[foox0==1])+lam.0*exp(b*b0)/(b*b1)*(exp(b*b1*tau1/2)-1)+lam.1*exp(b*b0)/(b*b1)*(exp(b*b1*s)-exp(b*b1*tau1/2)))
      }
    }
    Lam-a
  }
  # if a=0, then function Lam outputs the cumulative hazard up to time t
  # Lam0 is the total hazard up to tau
  #Lam0=Lam(tau)
  Lam0=Lam(cen)
  m <- rpois(1, Lam0)    
  event0 <- NULL
  X2 <- NULL
  if (m>0){
    for(kk in 1:m){
      u <- runif(1)*Lam0
      #foo<- uniroot(Lam, c(10^(-10),tau), a=u)$root
      foo<- uniroot(Lam, c(10^(-10),cen), a=u)$root
      event0<- c(event0, foo) 
      X2 <- c(X2, x[foo<t][1])
    }
    event <- event0[order(event0)]
    X2 <- X2[order(event0)]
  } else { event<- -99; X2 <- -99}
  X1 <- g.dat_am(event, b0, b1)	 
  t(matrix(cbind(k,event,X1,X2), ncol=4 )   )
}


#generate observations of covariate other than those at event times
gencarr_two_am_re <- function(k, data1, b0, b1, tau){
  tmp <- data1[data1$id==k,]
  vis <- c(0,(0:(tau-1))+runif(tau))
  #vis <- (0:(tau-1))+runif(tau)
  #vis <- c(0,runif(4),rep(1:(tau-1),5)+runif((tau-1)*5))
  #vis <- sort(vis)
  x1 <- g.dat_am(vis, b0, b1)
  x2 <- sapply(vis, 
               function(s, t=tmp$time, x=tmp$gas){ x[s<t][1] 
               })
  t(as.matrix(cbind(id=k, time=vis, z1=x1, z2=x2)))
} 


#mprob: missing probability(1, 0.8, 0.6, 0.4)
#it is possible that among N people, some don't have events at all.
sim.data_two_am_re <- function(N, beta,gamma, mprob, sce, tau,var,frailty)
{
  if (sce=="t")
  {
    Mu <- c(0.5, -0.05)
    Sigma <- c(0.05, 0.0005)
    #Mu <- c(1.5,-0.05)
    rr2 <- 6
  }
  if (sce=="nt")
  {
    Mu <- c(0.5, 0)
    Sigma <- c(0.05, 0.0005)
    #Mu <- c(1.5,0)
    rr2 <- 4
  }
  #tau <- 20
  
  b0 <- rnorm(N,mean=Mu[1],sd=sqrt(Sigma[1]))
  b1 <- rnorm(N,mean=Mu[2],sd=sqrt(Sigma[2]))
  
  #binary covariate
  gdata <- gencov_b_am(N=N, rr1=4, rr2=rr2, lc=1, tau=tau)
  
  g.censor = runif(N, 0, tau)
  tmp = NULL
  for (i in 1:N){
    #tmp <- cbind(tmp, genevent_two(i, data1=gdata, b0[i], b1[i],beta=beta,lam0=0.1,lam1=0.3, tau=tau))
    tmp <- cbind(tmp, genevent_two_am_re(i, data1=gdata, b0[i], b1[i],beta=beta,gamma=gamma,lam0=0.3,lam1=0.5, tau=tau, cen=g.censor[i],var=var,frailty=frailty))
  }	
  
  ##  tmp<-sapply(1:N, genevent,b0=2,b1=1,beta=0.5,lam0=0.05,lam1=0.2, tau=20)
  edata <- data.frame(matrix(unlist(tmp),ncol=4, byrow=TRUE, 
                             dimnames=list(NULL, c("id", "time","x","z")) ))
  edata <- edata[edata$time!=-99,]
  
  tmp =NULL
  for (i in 1:N){
    tmp <- cbind(tmp, gencarr_two_am_re(i, data1=gdata, b0[i], b1[i], tau=tau))
  }	 	
  cdata<-data.frame(matrix(unlist(tmp),ncol=4, byrow=TRUE, 
                           dimnames=list(NULL, c("id", "time","x","z")) ))
  
  b.phar <- edata
  #bindx <-  rbinom(5*N*tau,1,mprob)
  bindx <-  rbinom(N*(tau+1),1,mprob)
  #non-uniform missing probability
  #bindx[(1:(N*tau))%%20>10] <-rbinom(N*tau,1,mprob[bb]*0.5)
  bindx[0:(N-1)*(tau+1)+1] <- 1
  b.carr <- cdata[bindx ==1,]
  
  #censoring time is unif(10,20)
  #g.censor = rep(NA, N); for (i in 1:N){	g.censor[i] = max(b.carr$time[b.carr$id==i])} 
  #g.censor = runif(N, tau/4*3, tau) 
  #g.censor = runif(N, 0, tau)
  #b.phar$censor <- 1
  b.carr$censor <- 1
  for (i in 1:N){	b.carr$censor[b.carr$id==i][b.carr$time[b.carr$id==i]>g.censor[i]] <- 0}
  b.carr <- b.carr[b.carr$censor==1,]
  for (i in 1:N){	b.carr$cent[b.carr$id==i] <- rep(g.censor[i], nrow(b.carr[b.carr$id==i,]))}
  for (i in 1:N){	b.phar$cent[b.phar$id==i] <- rep(g.censor[i], nrow(b.phar[b.phar$id==i,]))}
  b.carr <- b.carr[,c("id","time","x","z","cent")]

  return(list(b.carr, b.phar))
}
